package de.ipk.agbi.flarex.client;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.rmi.RemoteException;

import javax.xml.rpc.ServiceFactory;
import javax.xml.rpc.Stub;

import de.ipk.agbi.flarex.client.proxy.FlarexMart_PortType;
import de.ipk.agbi.flarex.client.proxy.FlarexMart_Service;

public class FlarexMartClient {

	private FlarexMart_PortType binding;
	private String user = "radchuk";
	private String password = "pwd4debug!";
	
	private String url = "http://localhost:37900/FlarexMart/FlarexMart";
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		long start = System.currentTimeMillis();
		long heapallstart = Runtime.getRuntime().totalMemory();
		long heapfreestart = Runtime.getRuntime().freeMemory();
		System.out.println("Init - Free Mem = "+(heapfreestart/1024)+"KB \tAll Mem = "+(heapallstart/1024)+"KB");
		
		int testCase = 0;
		String option = "";
		FlarexMartClient client = new FlarexMartClient();
		int normid = 141;
		String embl = "AB063521";
		
		if(args.length<1){
			client.help();
			return;
		}
		for (int i = 0; i < args.length; i++) {
			String string = args[i];
			if (string.equalsIgnoreCase("-help")) {
				client.help();
				return;
			}
			if (string.equalsIgnoreCase("-case")) {
				testCase = Integer.parseInt(args[i+1]);				
			}
			if (string.equalsIgnoreCase("-option")) {
				option = args[i+1];				
			}
			if (string.equalsIgnoreCase("-user")) {
				client.setUser(args[i+1]);				
			}
			if (string.equalsIgnoreCase("-pass")) {
				client.setPassword(args[i+1]);
			}
		}
		switch (testCase) {
		case 1:
			if(option.equalsIgnoreCase(""))
				client.testCaseGeneIntens(normid);
			else
				client.testCaseGeneIntens(Integer.parseInt(option));
			break;
		case 2:
			if(option.equalsIgnoreCase(""))
				client.testCaseECMapping(embl);
			else
				client.testCaseECMapping(option);
			break;
		case 3:
			client.testCaseProvider();
			break;
		case 4:
			client.testCaseAllExperiment();
			break;
		case 5:
			client.testCaseAllNormalisationGeneWithEmblAndEC();
			break;
		case 6:
			if(option.equalsIgnoreCase(""))
				client.testCaseGOAnnotation(normid);
			else
				client.testCaseGOAnnotation(Integer.parseInt(option));
			break;
		default:
			break;
		}
		long heapfreeend = Runtime.getRuntime().freeMemory();
		long heapallend = Runtime.getRuntime().totalMemory();
		long end = System.currentTimeMillis();
		System.out.println("End - Time = "+(end-start)+" Free Mem = "+(heapfreeend/1024)+"KB \tAll Mem = "+(heapallend/1024)+"KB");
	}

	public FlarexMartClient() {
		try {
			ServiceFactory factory = ServiceFactory.newInstance();
			FlarexMart_Service mart = ((de.ipk.agbi.flarex.client.proxy.FlarexMart_Service)factory.loadService(de.ipk.agbi.flarex.client.proxy.FlarexMart_Service.class));					
			binding = mart.getFlarexMartServicePort();		
			((Stub) binding)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, url);			
		} catch (javax.xml.rpc.ServiceException jre) {
			if (jre.getLinkedCause() != null)
				jre.getLinkedCause().printStackTrace();
			throw new junit.framework.AssertionFailedError(
					"JAX-RPC ServiceException caught: " + jre);
		} 
	}

	public void testCaseGeneIntens(int id) {
		System.out.println("Method testGeneIntens with ID "+id);
		System.out.println("You work as user "+user);
		System.out.println("Please be patient. This could take a while.");
		System.out.println("Results are written into GeneIntensitiesForID_"+id+".txt");
		java.lang.String[][] value = null;
		Writer geneout;
		BufferedWriter genebuf;
		Writer expout;
		BufferedWriter expbuf;		
		try {
//			save gene intensities
			geneout = new FileWriter("GeneIntensitiesForID_"+id+".txt");
			genebuf = new BufferedWriter(geneout);
//			save exp details
			expout = new FileWriter("ExpDetailsForID_"+id+".txt");
			expbuf = new BufferedWriter(expout);
//			fetch intensities
			value = binding.getNormalizedGeneIntensities(id,user,password);
			for (int index = 0; index < value[0].length; index++) {						
				for (int index1 = 0; index1 < value.length; index1++) {
					String string = value[index1][index];
					genebuf.write(string+ "\t");
				}
				genebuf.newLine();
			}
////			fetch exp details
//			for (int index = 1; index < value.length; index++) {
////				System.out.println(value[index][0]);
//				String[][] label = binding.getLabeledProbeDetailsForProbeName(18,value[index][0],user,password);
//				for (int i = 0; i < label.length; i++) {
//					String[] strings = label[i];
//					for (int j = 0; j < strings.length; j++) {
//						expbuf.write(strings[j]+ "\t");
//					}
//					expbuf.newLine();
//				}
//			}
			expbuf.newLine();
			expbuf.write("Zusammenfassung");
			expbuf.newLine();
			String[][] exp = binding.getProbeDetailSummaryForExperiment(18,user,password);
			for (int i = 0; i < exp.length; i++) {
				String[] strings = exp[i];
				for (int j = 0; j < strings.length; j++) {
					expbuf.write(strings[j]+ "\t");
				}
				expbuf.newLine();
			}
			genebuf.flush();
			genebuf.close();
			expbuf.flush();
			expbuf.close();
		} catch (RemoteException e) {			
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void testCaseECMapping(String embl) {
		System.out.println("Method testECMapping with Embl ID = "+embl);
		System.out.println("You work as user "+user);
		System.out.println("Please be patient. This could take a while.");
		java.lang.String[] value = null;
		try {
			value = binding.getECForEmblID(embl, user,password);
			for (int i = 0; i < value.length; i++) {				
				System.out.println("element " + i + " = " + value[i]);
			}
		} catch (RemoteException e) {			
			e.printStackTrace();
		}
	}
	/**
	 * Print out all related information for every provider stored in the database
	 */
	public void testCaseProvider(){
		System.out.println("Method testGeneProvider");
		System.out.println("You work as user "+user);
		try {
			//fetch providerid's
			int[] provids = binding.getProviderIDs();
			for (int i = 0; i < provids.length; i++) {
				System.out.println("ID = "+provids[i]);				
				//fetch provider details
				String[][] detail = binding.getProviderDetails(provids[i]);
				for (int k = 0; k< detail.length; k++) {
					StringBuffer buf = new StringBuffer();
					String[] t = detail[0];
					for (int j = 0; j < t.length; j++) {
						buf.append(detail[k][j] + "\t");
					}
					System.out.println(buf);
				}
				//fetch experimentid's for provider
				int[] expids = binding.getExperimentIDsOfProvider(provids[i],user,password);
				System.out.println("Experiment ID's:");
				for (int k = 0; k< expids.length; k++) {
					System.out.println("-Exp = "+expids[k]);
					//fetch details for experimentid's
					String[][] expdetail = binding.getExperimentDetails(expids[k],user,password);					
					//print array
					for (int  l= 0; l< expdetail.length; l++) {
						StringBuffer buf = new StringBuffer();
						String[] t = expdetail[0];
						buf.append("\t");
						for (int m = 0; m < t.length; m++) {
							buf.append(expdetail[l][m] + "\t");
						}
						System.out.println(buf);
					}
					//fetch normalisationid's
					int[] normids = binding.getNormalisationIDsOfExperiment(expids[k],user,password);
					System.out.println("\tNormalisation ID's for Experiment:");
					for (int j = 0; j < normids.length; j++) {						
						System.out.println("\t\t ID = "+normids[j]);
						//fetch normalisationdetails for normalisation
						String[][] normDetail = binding.getNormalisationDetails(normids[j],user,password);
//						print array
						for (int  l= 0; l< normDetail.length; l++) {
							StringBuffer buf = new StringBuffer();
							String[] t = normDetail[0];
							buf.append("\t\t\t");
							for (int m = 0; m < t.length; m++) {
								buf.append(normDetail[l][m] + "\t");
							}
							System.out.println(buf);
						}						
					}
					System.out.println();
				}
				System.out.println("=========================================");
			}
		} catch (RemoteException e) {			
			e.printStackTrace();
		}
	}
	/**
	 * Print out all related information for every experiment stored in the database
	 */
	public void testCaseAllExperiment(){
		System.out.println("Method testCaseAllExperiments");
		System.out.println("You work as user "+user);
		try {			
			int[] expids = binding.getAllExperimentIDs(user,password);
			for (int k = 0; k< expids.length; k++) {
				System.out.println("-Exp = "+expids[k]);
				String[][] expdetail = binding.getExperimentDetails(expids[k],user,password);
//				print array
				for (int  l= 0; l< expdetail.length; l++) {
					StringBuffer buf = new StringBuffer();
					String[] t = expdetail[0];
					buf.append("\t");
					for (int m = 0; m < t.length; m++) {
						buf.append(expdetail[l][m] + "\t");
					}
					System.out.println(buf);
				}
				int[] hybids = binding.getHybridisationIDsOfExperiment(expids[k],user,password);
				System.out.println("\tHybridisation ID's for Experiment:");
//				print array
//				System.out.print("\t\t");
				for (int i = 0; i < hybids.length; i++) {
					System.out.println("\t\tHYB ID = "+hybids[i]);
					int[] labelprobe = binding.getProbeIDsForHybridisationID(hybids[i], user, password);
					for (int j = 0; j < labelprobe.length; j++) {
						System.out.println("\t\t\tProbe ID = "+labelprobe[j]);																
						String[][] probeDetail = binding.getProbeDetailsForProbeID(labelprobe[j],user,password);
//						print array
						System.out.println("\t\t\t\tDetails:");
						for (int  l= 0; l< probeDetail.length; l++) {
							StringBuffer buf = new StringBuffer();
							String[] t = probeDetail[0];
							buf.append("\t\t\t\t");
							for (int m = 0; m < t.length; m++) {
								buf.append(probeDetail[l][m] + "\t");
							}
							System.out.println(buf);
						}
						
					}
				}
				System.out.println();
				int[] normids = binding.getNormalisationIDsOfExperiment(expids[k],user,password);
				System.out.println("\tNormalisation ID's for Experiment:");
//				print array
				for (int j = 0; j < normids.length; j++) {						
					System.out.println("\t\t ID = "+normids[j]);
					// Normalisation Detail
					String[][] normDetail = binding.getNormalisationDetails(normids[j],user,password);
//					print array
					for (int  l= 0; l< normDetail.length; l++) {
						StringBuffer buf = new StringBuffer();
						String[] t = normDetail[0];
						buf.append("\t\t\t");
						for (int m = 0; m < t.length; m++) {
							buf.append(normDetail[l][m] + "\t");
						}
						System.out.println(buf);
					}					
				}
				String[][] labeldetail = binding.getProbeDetailSummaryForExperiment(expids[k],user,password);
				System.out.println("\tLabeledprobe details for experiment: "+expids[k]);
				for (int i = 0; i < labeldetail.length; i++) {
					String[] strings = labeldetail[i];
					System.out.print("\t\t");
					for (int j = 0; j < strings.length; j++) {
						System.out.print(labeldetail[i][j]+"\t");
					}
					System.out.println();
				}
				System.out.println("\tReplicagroups for experiment: "+expids[k]);
				int[][] replica = binding.getReplicaGroupsOfExperiment(expids[k],user,password);
				for (int i = 0; i < replica.length; i++) {
					int[] js = replica[i];
					System.out.print("\t\tGroup = ");
					for (int j = 0; j < js.length; j++) {
						System.out.print(replica[i][j]+"\t");
					}
					System.out.println();
				}
				System.out.println();
				System.out.println("=========================================");
				System.out.println();
			}			
		} catch (RemoteException e) {			
			e.printStackTrace();
		}
	}
	public void testCaseGOAnnotation(int id){
		System.out.println("Method testGOAnnotation with ID "+id);
		System.out.println("You work as user "+user);
		System.out.println("Please be patient. This could take a while.");
		System.out.println("Results are written into GOACCForNormID_"+id+".txt");
		java.lang.String[][] value = null;
		Writer geneout;
		BufferedWriter genebuf;		
		try {
//			save gene intensities
			geneout = new FileWriter("GOACCForNormID_"+id+".txt");
			genebuf = new BufferedWriter(geneout);
			//fetch intensities
			value = binding.getNormalizedGeneIntensities(id,user,password);
			String[][] go = binding.getGOAccForBiosequences(value[0]);
			for (int index = 0; index < go.length; index++) {
				String[] strings =go[index];
				for (int index1 = 0; index1 < strings.length; index1++) {
					String string = strings[index1];
					if(string.length()>1)
						genebuf.write(string+";");
				}
				genebuf.newLine();
			}	
			genebuf.flush();
			genebuf.close();
		} catch (RemoteException e) {			
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	/**
	 * This testCase fetch for one normalisation the geneintensities and save them
	 * into a file GeneIntensities.txt. After that for every gene the embl-id 
	 * and ec-number will be store to files. 
	 */
	public void testCaseAllNormalisationGeneWithEmblAndEC(){
		System.out.println("Method testCaseAllNormalisationGeneWithEmblAndEC");
		System.out.println("You work as user "+user);
		System.out.println("Please be patient. This could take a while.");		
		int[] expids;
		Writer geneout;
		BufferedWriter genebuf;
		Writer ecout;
		BufferedWriter ecbuf;
		Writer emblout;
		BufferedWriter emblbuf;
		Writer expout;
		BufferedWriter expbuf;		
		try {
			expids = binding.getAllExperimentIDs(user,password);
			//save gene intensities
			geneout = new FileWriter("GeneIntensities.txt");
			genebuf = new BufferedWriter(geneout);
			//save ec mapping
			ecout = new FileWriter("ecmapping.txt");
			ecbuf = new BufferedWriter(ecout);
			//save embl mapping
			emblout = new FileWriter("emblmapping.txt");
			emblbuf = new BufferedWriter(emblout);
//			save exp details
			expout = new FileWriter("ExpDetailsForID.txt");
			expbuf = new BufferedWriter(expout);
			for (int i = 0; i < expids.length; i++) {
				int[] normids = binding.getNormalisationIDsOfExperiment(expids[i],user,password);				
				genebuf.write("NEW Exp ID = "+expids[i]);
				ecbuf.write("NEW Exp ID = "+expids[i]);
				emblbuf.write("NEW Exp ID = "+expids[i]);
				genebuf.write("====================== NEW NORMALISATION ID ========================");
				genebuf.newLine();
				//save meta information
				for (int j = 0; j < normids.length; j++) {
					
					genebuf.write("Normalisation ID = "+normids[j]);					
					ecbuf.write("Normalisation ID = "+normids[j]);					
					emblbuf.write("Normalisation ID = "+normids[j]);					
					genebuf.newLine();
					ecbuf.newLine();
					emblbuf.newLine();
					
					System.out.println("Get Normalised Gene's for Experiment:"+expids[i]+" and Normalisation:"+normids[j]);
					int k = normids[j];
					String value[][] = binding.getNormalizedGeneIntensities(k,user,password);
//					save to file gene intensities
					for (int index = 0; index < value[0].length; index++) {						
						for (int index1 = 0; index1 < value.length; index1++) {
							String string = value[index1][index];
							genebuf.write(string+ "\t");
						}
						genebuf.newLine();
					}					
					
					String[] t = value[0];
					
					String[][] ec = binding.getECForBiosequences(t,user,password);					
//					save to file ec ids
					for (int index = 0; index < ec.length; index++) {
						String[] strings = ec[index];
						for (int index1 = 0; index1 < strings.length; index1++) {
							String string = strings[index1];
							if(string.length()>1)
								ecbuf.write(string+";");
						}
						ecbuf.newLine();
					}
					ec = null;
					System.gc();
					
					String[][] embl = binding.getEmblForBiosequences(t,user,password);
//					save to file embl ids
					for (int index = 0; index < embl.length; index++) {
						String[] strings = embl[index];
						for (int index1 = 0; index1 < strings.length; index1++) {
							String string = strings[index1];
							if(string.length()>1)
								emblbuf.write(string+";");
						}
						emblbuf.newLine();
					}
					embl = null;
					System.gc();					
				}
//				save to file exp details
				expbuf.write("Exp ID = "+expids[i]);				
				String[][] exp = binding.getProbeDetailSummaryForExperiment(expids[i],user,password);
				for (int j = 0; j < exp.length; j++) {
					String[] strings = exp[j];
					for (int k = 0; k < strings.length; k++) {
						expbuf.write(strings[k]+ "\t");
					}
					expbuf.newLine();
				}
				expbuf.newLine();
			}
			genebuf.flush();
			genebuf.close();
			expbuf.flush();
			expbuf.close();
			ecbuf.flush();
			ecbuf.close();
			emblbuf.flush();
			emblbuf.close();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (IOException e1) {
			e1.printStackTrace();
		}		
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getUser() {
		return user;
	}

	public void setUser(String user) {
		this.user = user;
	}
	
	public void help(){
		System.out.println("Usage: java -jar FlarexMartClient.jar [-help] -case NumCase " +
				"[-option inputParameter] -user username -pass password");
		System.out.println("Values for NumCase are:");
		System.out.println("\t1 = testCaseGeneIntens with inputParameter normalisationID");
		System.out.println("\t2 = testCaseECMapping with inputParameter Embl-ID");
		System.out.println("\t3 = testCaseProvider has no inputParameter");
		System.out.println("\t4 = testCaseAllExperiment has no inputParameter");
		System.out.println("\t5 = testCaseAllNormalisationGeneWithEmblAndEC has no inputParameter");
		System.out.println("\t6 = testCaseGOAnnotation(int) with inputParameter normalisationID");
	}
	
}
